var d3 = require('d3');
var BaseWidget = require('@domoinc/base-widget');

module.exports = BaseWidget.extend('LiquidFillGauge', {
    initialize: function () {
        var _Chart = this;
        _Chart._newConfig = {
            width: {
                name: 'Widget Width',
                description: 'Width of the widget',
                value: 400,
                type: 'number',
                units: 'px'
            },
            height: {
                name: 'Widget Height',
                description: 'Height of the widget',
                value: 400,
                type: 'number',
                units: 'px'
            },
            chartName: {
                description: 'Name of chart for reporting',
                type: 'string',
                value: 'LiquidFillGauge'
            }, 
            minValue: {
                name: 'The gauge minimum value',
                description: 'The gauge minimum value',
                value: 0,
                type: 'number'
            },
            maxValue: {
                name: 'The gauge maximum value',
                description: 'The gauge maximum value',
                value: 100,
                type: 'number'
            },
            circleThickness: {
                name: 'The gauge maximum value',
                description: 'The gauge maximum value',
                value: 0.05,
                type: 'number'
            },
            circleFillGap: {
                name: 'The gauge maximum value',
                description: 'The gauge maximum value',
                value: 0.05,
                type: 'number'
            },
            circleColor: {
                name: 'The gauge maximum value',
                description: 'The gauge maximum value',
                value: "#178BCA",
                type: 'string'
            },
            waveHeight: {
                name: 'The gauge maximum value',
                description: 'The gauge maximum value',
                value: 0.18,
                type: 'number'
            },
            waveCount: {
                name: 'The gauge maximum value',
                description: 'The gauge maximum value',
                value: 1,
                type: 'number'
            },
            waveRiseTime: {
                name: 'The gauge maximum value',
                description: 'The gauge maximum value',
                value: 1000,
                type: 'number'
            },
            waveAnimateTime: {
                name: 'The gauge maximum value',
                description: 'The gauge maximum value',
                value: 18000,
                type: 'number'
            },
            waveRise: {
                name: 'The gauge maximum value',
                description: 'The gauge maximum value',
                value: true,
            },
            waveHeightScaling: {
                name: 'The gauge maximum value',
                description: 'The gauge maximum value',
                value: true,
            },
            waveAnimate: {
                name: 'The gauge maximum value',
                description: 'The gauge maximum value',
                value: true,
            },
            waveColor: {
                name: 'The gauge maximum value',
                description: 'The gauge maximum value',
                value: "#178BCA",
                type: "string"
            },
            waveOffset: {
                name: 'The gauge maximum value',
                description: 'The gauge maximum value',
                value: 0,
                type: 'number'
            },
            textVertPosition: {
                name: 'The gauge maximum value',
                description: 'The gauge maximum value',
                value: .5,
                type: 'number'
            },
            textSize: {
                name: 'The gauge maximum value',
                description: 'The gauge maximum value',
                value: 1,
                type: 'number'
            },
            valueContUp: {
                name: 'The gauge maximum value',
                description: 'The gauge maximum value',
                value: true,
            },
            displayPercent: {
                name: 'The gauge maximum value',
                description: 'The gauge maximum value',
                value: false
            },
            textColor: {
                name: 'The gauge maximum value',
                description: 'The gauge maximum value',
                value: "#A4DBf8",
                type: 'string'
            },
            waveTextColor: {
                name: 'The gauge maximum value',
                description: 'The gauge maximum value',
                value: "#A4DBf8",
                type: 'string'
            },
            elementId: {
                description: 'Element id',
                value: '',
            }
        };
        _Chart.mergeConfig(_Chart._newConfig);

        var superTransform = _Chart.transform;
        _Chart.transform = function (data) {
            init(data);
            return data;
        };

        function init(validData) {
            var elementId = _Chart.c('elementId')
            var radius = Math.min(_Chart.c('width'), _Chart.c('height')) / 2;
            var locationX = parseInt(_Chart.c('width')) / 2 - radius;
            var locationY = parseInt(_Chart.c('height')) / 2 - radius;
            var fillPercent = Math.max(_Chart.c('minValue'), Math.min(_Chart.c('maxValue'), validData)) / _Chart.c('maxValue');
            var waveHeightScale = d3.scale.linear().range([0, _Chart.c('waveHeight'),0]).domain([0,50,100]);

            var textPixels = (radius / 2);
            var textFinalValue = parseFloat(validData).toFixed(2);
            var textStartValue = _Chart.c('minValue');
            var percentText = "";
            // var percentText = _Chart.c('displayPercent') ? "%" : "";
            var circleThickness = _Chart.c('circleThickness') * radius;
            var circleFillGap = _Chart.c('circleFillGap') * radius;
            var fillCircleMargin = circleThickness + circleFillGap;
            var fillCircleRadius = radius - fillCircleMargin;
            var waveHeight = fillCircleRadius * waveHeightScale(fillPercent * 100);

            var waveLength = fillCircleRadius * 2 / _Chart.c('waveCount');
            var waveClipCount = 1 + _Chart.c('waveCount');
            var waveClipWidth = waveLength * waveClipCount;

            // Rounding functions so that the correct number of decimal places is always displayed as the value counts up.
            var textRounder = function(value) {
                return Math.round(value);
            };
            
            if (parseFloat(textFinalValue) != parseFloat(textRounder(textFinalValue))) {
                textRounder = function(value) {
                    return parseFloat(value).toFixed(1);
                };
            }

            if (parseFloat(textFinalValue) != parseFloat(textRounder(textFinalValue))) {
                textRounder = function(value) {
                    return parseFloat(value).toFixed(2);
                };
            }

            // Data for building the clip wave area.
            var data = [];
            for(var i = 0; i <= 40 * waveClipCount; i++) {
                data.push({
                    x: i / (40 * waveClipCount),
                    y: (i / (40))
                });
            }

            // Scales for drawing the outer circle.
            var gaugeCircleX = d3.scale.linear().range([0, 2 * Math.PI]).domain([0, 1]);
            var gaugeCircleY = d3.scale.linear().range([0, radius]).domain([0, radius]);

            // Scales for controlling the size of the clipping path.
            var waveScaleX = d3.scale.linear().range([0, waveClipWidth]).domain([0, 1]);
            var waveScaleY = d3.scale.linear().range([0, waveHeight]).domain([0, 1]);

            // Scales for controlling the position of the clipping path.
            var waveRiseScale = d3.scale.linear()
                // The clipping area size is the height of the fill circle + the wave height, so we position the clip wave
                // such that the it will overlap the fill circle at all when at 0%, and will totally cover the fill
                // circle at 100%.
                .range([(fillCircleMargin+fillCircleRadius*2+waveHeight),(fillCircleMargin-waveHeight)])
                .domain([0,1]);

            var waveAnimateScale = d3.scale.linear()
                .range([0, waveClipWidth-fillCircleRadius*2]) // Push the clip area one full wave then snap back.
                .domain([0,1]);

            // Scale for controlling the position of the text within the gauge.
            var textRiseScaleY = d3.scale.linear()
                .range([fillCircleMargin+fillCircleRadius*2,(fillCircleMargin+textPixels*0.7)])
                .domain([0,1]);

            // Center the gauge within the parent SVG.
            var gaugeGroup = d3.select(elementId).append("g")
                .attr('transform','translate('+locationX+','+locationY+')');

            // Draw the outer circle.
            var gaugeCircleArc = d3.svg.arc()
                .startAngle(gaugeCircleX(0))
                .endAngle(gaugeCircleX(1))
                .outerRadius(gaugeCircleY(radius))
                .innerRadius(gaugeCircleY(radius-circleThickness));
            gaugeGroup.append("path")
                .attr("d", gaugeCircleArc)
                .style("fill", _Chart.c('circleColor'))
                .attr('transform','translate('+radius+','+radius+')');

            // Text where the wave does not overlap.
            var text1 = gaugeGroup.append("text")
                .text(textRounder(textStartValue) + percentText)
                .attr("class", "liquidFillGaugeText")
                .attr("text-anchor", "middle")
                .attr("font-size", textPixels + "px")
                .style("fill", _Chart.c('textColor'))
                .attr('transform','translate('+radius+','+textRiseScaleY(_Chart.c('textVertPosition'))+')');

            // The clipping wave area.
            var clipArea = d3.svg.area()
                .x(function(d) { return waveScaleX(d.x); } )
                .y0(function(d) { return waveScaleY(Math.sin(Math.PI*2*_Chart.c('waveOffset')*-1 + Math.PI*2*(1-_Chart.c('waveCount')) + d.y*2*Math.PI));} )
                .y1(function(d) { return (fillCircleRadius*2 + waveHeight); } );
            var waveGroup = gaugeGroup.append("defs")
                .append("clipPath")
                .attr("id", "clipWave" + this.el);
            var wave = waveGroup.append("path")
                .datum(data)
                .attr("d", clipArea)
                .attr("T", 0);

            // The inner circle with the clipping wave attached.
            var fillCircleGroup = gaugeGroup.append("g")
                .attr("clip-path", "url(#clipWave" + this.el + ")");
            fillCircleGroup.append("circle")
                .attr("cx", radius)
                .attr("cy", radius)
                .attr("r", fillCircleRadius)
                .style("fill", _Chart.c('waveColor'));

            // Text where the wave does overlap.
            var text2 = fillCircleGroup.append("text")
                .text(textRounder(textStartValue) + percentText)
                .attr("class", "liquidFillGaugeText")
                .attr("text-anchor", "middle")
                .attr("font-size", textPixels + "px")
                .style("fill", _Chart.c('waveTextColor'))
                .attr('transform','translate('+radius+','+textRiseScaleY(_Chart.c('textVertPosition'))+')');

            // Make the value count up.
            if(_Chart.c('valueContUp')){
                var textTween = function(){
                    var i = d3.interpolate(this.textContent, textFinalValue);
                    return function(t) { this.textContent = textRounder(i(t)) + percentText; }
                };
                text1.transition()
                    .duration(_Chart.c('waveRiseTime'))
                    .tween("text", textTween);
                text2.transition()
                    .duration(_Chart.c('waveRiseTime'))
                    .tween("text", textTween);
            }

            // Make the wave rise. wave and waveGroup are separate so that horizontal and vertical movement can be controlled independently.
            var waveGroupXPosition = fillCircleMargin+fillCircleRadius*2-waveClipWidth;
            if(_Chart.c('waveRise')){
                waveGroup.attr('transform','translate('+waveGroupXPosition+','+waveRiseScale(0)+')')
                .transition()
                .duration(_Chart.c('waveRiseTime'))
                .attr('transform','translate('+waveGroupXPosition+','+waveRiseScale(fillPercent)+')')
                .each("start", function(){ wave.attr('transform','translate(1,0)'); }); // This transform is necessary to get the clip wave positioned correctly when waveRise=true and waveAnimate=false. The wave will not position correctly without this, but it's not clear why this is actually necessary.
            } else {
                waveGroup.attr('transform','translate('+waveGroupXPosition+','+waveRiseScale(fillPercent)+')');
            }

            if(_Chart.c('waveAnimate')) animateWave();

            function animateWave() {
                wave.attr('transform','translate('+waveAnimateScale(wave.attr('T'))+',0)');
                wave.transition()
                .duration(_Chart.c('waveAnimateTime') * (1-wave.attr('T')))
                .ease('linear')
                .attr('transform','translate('+waveAnimateScale(1)+',0)')
                .attr('T', 1)
                .each('end', function(){
                    wave.attr('T', 0);
                    animateWave(_Chart.c('waveAnimateTime'));
                });
            }
        }
    }
});
