var blue = '\x1B[0;36m';
var yellow = '\x1B[1;33m';
var noColor = '\x1B[0m';

var request = require('request');
var fs = require('fs');

var apiUrl = 'http://domoapps.dev.domo.com/devapi/';
var sandbox = true;

module.exports = function(grunt) {
    'use strict';
    var svgs = grunt.file.expand('*.svg');
    var widget;

    var categories = [
        {name: 'Vertical Bar Charts'},
        {name: 'Horizontal Bar Charts'},
        {name: 'Line & Area Charts'},
        {name: 'Pie & Funnel Charts'},
        {name: 'Maps'},
        {name: 'Gauges'},
        {name: 'Bubble & Scatter Charts'},
        {name: 'Other Charts'}
    ];

    // Project configuration
    grunt.initConfig({
        // Metadata
        bower: grunt.file.readJSON('bower.json'),
        banner: '/*! <%= bower.name %> - v<%= bower.version %> - ' +
            '<%= grunt.template.today("yyyy-mm-dd") %>\n' +
            '* Copyright (c) <%= grunt.template.today("yyyy") %> Domo */\n',
        // Task configuration
        portPick: {
      livereload: {
        options: {
          port: 1337,
          name: 'livereload'
        },
        targets: [
          'watch.options.livereload',
          'connect.options.livereload',
          'connect.dist.options.livereload',
          'connect.open.options.livereload'
        ]
      },
      server: {
        options: {
          port: 9000,
          name: 'server'
        },
        targets: [
          'connect.options.port',
          'autoshot.default_options.options.local.port'
        ]
      }
    },
        jshint: {
            dev: {
                options: {
                    force: true,
                    jshintrc: '.jshintrc',
                    reporter: require('jshint-stylish')
                },
                src: ['*.js', '!Gruntfile.js']
            },
            dep: {
                options: {
                    jshintrc: '.jshintrc',
                    reporter: require('jshint-stylish')
                },
                src: ['*.js', '!Gruntfile.js']
            }
        },
        watch: {
            options: {
                livereload: 0,
            },
            all: {
                files: ['*.js'],
                tasks: ['jshint:dev', 'bowerInstall'],
                options: {
                    spawn: true,
                    reload: true
                }
            },
        },
        connect: {
            options: {
                port: 0,
                open: false,
                // Change this to '0.0.0.0' to access the server from outside
                hostname: 'localhost'
            },
            dist: {
                options: {
                    livereload: 0
                }
            },
            open: {
                options: {
                    livereload: 0,
                    open: true
                }
            }
        },
        bowerInstall: {
            target: {
                src: [
                    'index.html',
                    'auto.html'
                ],
            }
        },
        init: {
            dist: {
                title: "<%= bower.name %>"
            }
        },
        release: {
            options: {
                file: 'bower.json', //default: package.json
                additionalFiles: ['package.json'],
                npm: false, //default: true
                npmtag: false, //default: no tag
                tagName: 'v<%= version %>'
            }
        },
        prompt: {
            config: {
                options: {
                    questions: [{
                        config: 'publishConfig.name',
                        type: 'input',
                        message: 'Widget name',
                        default: function() {
                             if (grunt.config('publishConfig') && grunt.config('publishConfig').name) {
                                return grunt.config('publishConfig')
                                    .name;
                            }
                            return grunt.config('bower')
                                .name;
                        }
                    },
                    {
                        config: 'publishConfig.version',
                        type: 'input',
                        message: 'Widget version',
                        default: function() {
                            return grunt.config('bower')
                                .version;
                        }
                    },
                    {
                        config: 'publishConfig.github',
                        type: 'input',
                        message: 'Github repo name',
                        default: function() {
                            if (grunt.config('publishConfig') && grunt.config('publishConfig')
                                .github) {
                                return grunt.config('publishConfig')
                                    .github;
                            }
                            var dir = __dirname.split('/');
                            var folder = dir[dir.length - 1];
                            return folder;
                        }
                    },
                    {
                        config: 'publishConfig.description',
                        type: 'input',
                        message: 'Enter a description',
                        default: function() {
                            if (grunt.config('publishConfig') && grunt.config('publishConfig').description) {
                                return grunt.config('publishConfig')
                                    .description;
                            }
                            return '';
                        }
                    },
                    {
                        config: 'publishConfig.tags',
                        type: 'input',
                        message: 'Enter a comma-seperated list of tags',
                        default: function() {
                            if (grunt.config('publishConfig') && grunt.config('publishConfig').tags) {
                                return grunt.config('publishConfig')
                                    .tags.join(',');
                            }
                            return '';
                        }
                    },
                    {
                        config: 'publishConfig.categories',
                        type: 'checkbox',
                        message: 'Select all categories that apply',
                        choices: categories
                    },
                    {
                        config: 'publishConfig.file',
                        type: 'list',
                        message: 'Choose the widget svg',
                        choices: svgs
                    }]
                }
            },
            chooseWidget: {
                options: {
                    questions: [{
                        config: 'publishConfigFile',
                        type: 'list',
                        message: 'Choose the widget to publish'
                    }]
                }
            }
        },
        autoshot: {
            default_options: {
                options: {
                    // necessary config
                    path: 'img/',
                    remote: {
                        files: []
                    },
                    local: {
                        path: '/',
                        port: 9001,
                        files: [
                            {
                                src: 'index.html',
                                dest: 'widget.png',
                                delay: 1000
                            }
                        ]
                    },
                    viewport: ['728x728']
                }
            }
        }
    });

    // These plugins provide necessary tasks
    grunt.loadNpmTasks('grunt-contrib-jshint');
    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.loadNpmTasks('grunt-contrib-connect');
    grunt.loadNpmTasks('grunt-bower-install');
    grunt.loadNpmTasks('grunt-release');
    grunt.loadNpmTasks('grunt-notify');
    grunt.loadNpmTasks('grunt-prompt');
    grunt.loadNpmTasks('grunt-autoshot');
    grunt.loadNpmTasks('grunt-port-pick');


    grunt.task.registerMultiTask('init', 'A task that creates a github repo.', function() {
        var shell = require('shelljs');

        shell.exec('git init');
        grunt.log.write(blue + 'Ran init.\n' + noColor);

        shell.exec('git add .');
        grunt.log.write(blue + 'Added all files.\n' + noColor);

        shell.exec('git commit -m "First Commit"');
        grunt.log.write(blue + 'Local commit was run successfully.\n' + noColor);

        grunt.log.write(blue + "Creating remote Repo: " + yellow + ('GITHUB_HOST=git.empdev.domo.com hub create DA/' + this.data.title) + noColor + "\n");
        shell.exec('GITHUB_HOST=git.empdev.domo.com hub create DA/' + this.data.title);
        grunt.log.write(blue + 'Created the repo!\n' + noColor);

        shell.exec('git push origin master');
        grunt.log.write(blue + 'Pushed to origin master\n' + noColor);

        shell.exit(1);
        grunt.log.write('Now run grunt deploy\n');
    });

    //Run Server
    grunt.registerTask('default', ['jshint:dev', 'bowerInstall', 'portPick:livereload', 'portPick:server', 'connect:dist', 'watch']);
    grunt.registerTask('server', ['jshint:dev', 'bowerInstall', 'portPick:livereload', 'portPick:server', 'connect:open', 'watch']);

    // Deploy to Github with tags
    grunt.registerTask('push', function(release){
        if (release) {
            release = release.toLowerCase();
        }
        if (release === 'patch') {
            grunt.task.run(['jshint:dep', 'release:patch']);
        } else if (release === 'minor') {
            grunt.task.run(['jshint:dep', 'release:minor']);
        } else if (release === 'major') {
            grunt.task.run(['jshint:dep', 'release:major']);
        } else {
            console.log('Use one of the following commands to push');
            console.log('grunt push:major');
            console.log('grunt push:minor');
            console.log('grunt push:patch');
        }
    })

    grunt.registerTask('saveWidget', function(){
        var publishConfig = grunt.config('publishConfig');
        var configFile = grunt.config('publishConfigFile');
        if(!configFile){
            configFile = 'publishConfig-' + publishConfig.name.replace(/\s/g, '') + '.json';
            grunt.config('publishConfigFile', configFile);
        }
        if(typeof publishConfig.tags === 'string'){
            publishConfig.tags = publishConfig.tags.split(/\s*,\s*/);
        }
        grunt.config('publishConfig', publishConfig);
        var fileName = publishConfig.file;
        delete publishConfig.file;
        var done = this.async();
        var id = publishConfig._id;
        var prodId = id;
        var devId = publishConfig._devid;
        if (sandbox) {
            id = devId;
        }
        if (id) {
            request({
                method: 'GET',
                url: apiUrl + '/widgets/' + id,
            }, function(error, response, body) {
                if (!error && response.statusCode === 200) {
                    updateWidget(publishConfig, id);
                } else {
                    postWidget(publishConfig);
                }
            })
        } else {
            postWidget(publishConfig);
        }

        function postWidget(widget) {
            var id = widget._id;
            request({
                method: 'POST',
                url: apiUrl + '/widgets',
                json: publishConfig
            }, function(error, response, body) {
                if (!error && response.statusCode === 200) {
                    console.log('Widget saved');
                    uploadFile(body);
                    delete body.updated;
                    if (sandbox) {
                        body._devid = body._id;
                        body._id = prodId;
                    } else {
                        body._devid = devId;
                    }
                    grunt.file.write(configFile, JSON.stringify(body));
                } else {
                    console.log('Error while saving');
                    done();
                }
            });
        }

        function updateWidget(widget, id) {
            request({
                method: 'PUT',
                url: apiUrl + '/widgets/' + id,
                json: widget
            }, function(error, response, body) {
                if (!error && response.statusCode === 200) {
                    console.log('Widget updated');
                    uploadFile(body);
                    delete body.updated;
                    body._id = prodId;
                    body._devid = devId;
                    grunt.file.write(configFile, JSON.stringify(body));
                } else {
                    console.log('Error while updating');
                    done();
                }
            });
        }

        function uploadFile(widget) {
            if (fileName && grunt.file.exists(fileName)) {
                var r = request.post(apiUrl + 'widgets/' + widget._id + '/image', function optionalCallback(err, httpResponse, body) {
                    if (err) {
                        done();
                        return console.error('upload failed:', err);
                    }
                    done();
                    console.log('Upload successful!');
                });
                var form = r.form();
                form.append('image', fs.createReadStream(fileName));

            } else {
                console.log('No svg found')
                done();
            }
        }
    });

    grunt.registerTask('readConfig', function(){
         var configFile = grunt.config('publishConfigFile');
         if (grunt.file.exists(configFile)) {
            grunt.config('publishConfig', grunt.file.readJSON(configFile));
            categories.map(function(category){
                var index = grunt.config('publishConfig').categories.indexOf(category.name);
                category.checked = index >= 0;
            });
        }
    });

    grunt.registerTask('publish', function(env, widgetName) {
        if(env && env.toLowerCase() === 'prod'){
            apiUrl = 'http://domoapps.dev.domo.com/api/';
            sandbox = false;
        } else if (env && !widgetName) {
            // If the keyword 'prod' is not used, assume the first argument was the widget name
            widgetName = env;
        }
        if(widgetName){
            var configFile = 'publishConfig-' + widgetName + '.json';
            grunt.config('publishConfigFile', configFile);
            grunt.task.run(['readConfig', 'prompt:config', 'saveWidget']);
        } else {
            var configs = grunt.file.expand('publishConfig-*.json');
            if(configs.length === 0){
                grunt.task.run(['prompt:config', 'saveWidget']);
            } else if(configs.length === 1) {
                var configFile = configs[0];
                grunt.config('publishConfigFile', configFile)
                grunt.task.run(['readConfig', 'prompt:config', 'saveWidget']);
            } else {
                var questions = grunt.config('prompt.chooseWidget.options.questions');
                questions[0].choices = configs;
                grunt.config('prompt.chooseWidget.options.questions', questions);
                grunt.task.run(['prompt:chooseWidget', 'readConfig', 'prompt:config', 'saveWidget']);
            }
        }
    });
    grunt.registerTask('unpublish', function(env, widgetName) {
        if(env && env.toLowerCase() === 'prod'){
            apiUrl = 'http://domoapps.dev.domo.com/api/';
            sandbox = false;
        } else if (env && !widgetName) {
            // If the keyword 'prod' is not used, assume the first argument was the widget name
            widgetName = env;
        }
        if(widgetName){
            var configFile = 'publishConfig-' + widgetName + '.json';
            grunt.config('publishConfigFile', configFile);
            grunt.task.run(['deleteWidget']);
        } else {
            var configs = grunt.file.expand('publishConfig-*.json');
            if(configs.length === 0){
                console.log('There are no widgets to unpublish');
            } else if(configs.length === 1) {
                var configFile = configs[0];
                grunt.config('publishConfigFile', configFile)
                grunt.task.run(['deleteWidget']);
            } else {
                var questions = grunt.config('prompt.chooseWidget.options.questions');
                questions[0].choices = configs;
                grunt.config('prompt.chooseWidget.options.questions', questions);
                grunt.task.run(['prompt:chooseWidget', 'deleteWidget']);
            }
        }
    });

    grunt.registerTask('deleteWidget', function(){
        var configFile = grunt.config('publishConfigFile');
            if (grunt.file.exists(configFile)) {
                var publishConfig = grunt.file.readJSON(configFile);
                var id = sandbox ? publishConfig._devid : publishConfig._id;
                if(id){
                    var done = this.async();
                    request({
                        method: 'delete',
                        url: apiUrl + '/widgets/' + id
                    }, function(error, response, body) {
                        if (!error) {
                            if (response.statusCode === 200) {
                                console.log('Widget unpublished');
                            } else if (response.statusCode === 404) {
                                console.log('Widget already unpublished');
                            }
                            if (sandbox) {
                                delete publishConfig._devid;
                            } else {
                                delete publishConfig._id;
                            }
                            grunt.file.write(configFile, JSON.stringify(publishConfig));
                        } else {
                            console.log('Error removing widget. Success unknown.');
                        }
                        done();
                    });
                } else {
                    console.log('This widget has not been published');
                }

            } else {
                console.log('This widget has not been published');
            }
    });
};