define([
            'jquery',
            'underscore',
            'api/SplunkVisualizationBase',
            'api/SplunkVisualizationUtils',
            'd3',
            'liquid_gauge'
        ],
        function(
            $,
            _,
            SplunkVisualizationBase,
            SplunkVisualizationUtils,
            d3
        ) {
  
    return SplunkVisualizationBase.extend({
 
        initialize: function() {
            // Save this.$el for convenience
            this.$el = $(this.el);
             
            // Add a css selector class
            this.$el.addClass('splunk-liquid-gauge');

        },
 
        getInitialDataParams: function() {
            return ({
                outputMode: SplunkVisualizationBase.ROW_MAJOR_OUTPUT_MODE,
                count: 10000
            });
        },

        formatData: function(data, config) {
            if(data.rows.length < 1) {
                return false;
            }

            var datum = SplunkVisualizationUtils.escapeHtml(parseFloat(data.rows[0][0]));

            // Check for invalid data
            if(_.isNaN(datum)) {
                throw new SplunkVisualizationBase.VisualizationError(
                    'This guage only supports numbers'
                );
            }
     
            return datum;
        },
  
        updateView: function(data, config) {
            // Guard for empty data
            if (!data) {
                return;
            }

            // Clear the div
            this.$el.empty();

            // Set height and width
            var height = 200;
            var width = 200;

            // Set min and max values
            var minValue = parseInt(config[this.getPropertyNamespaceInfo().propertyNamespace + 'minValue']) || 0;
            var maxValue = parseInt(config[this.getPropertyNamespaceInfo().propertyNamespace + 'maxValue']) || 1000;

            // Set wave settings
            var waveCount = parseInt(config[this.getPropertyNamespaceInfo().propertyNamespace + 'waveCount']) || 2;
            var waveColor = config[this.getPropertyNamespaceInfo().propertyNamespace + 'waveColor'] || '#178BCA';
            
            // Set circle color
            var circleColor = config[this.getPropertyNamespaceInfo().propertyNamespace + 'circleColor'] || '#178BCA';
            
            // Set text color
            var textColor = config[this.getPropertyNamespaceInfo().propertyNamespace + 'textColor'] || '#A4DBf8';
            var waveTextColor = config[this.getPropertyNamespaceInfo().propertyNamespace + 'waveTextColor'] || '#A4DBf8';

            // SVG setup
            var svg = d3.select(this.el).append('svg')
                .attr({
                    id: 'gauge',
                    height: height + 'px',
                    width: width + 'px',
                })
                .chart('LiquidFillGauge')
                .c({
                    height: height,
                    width: width,
                    elementId: '#gauge',
                    minValue: minValue,
                    maxValue: maxValue,
                    waveCount: waveCount,
                    waveColor: waveColor,
                    circleColor: circleColor,
                    textColor: textColor,
                    waveTextColor: waveTextColor
                })

            svg.draw(data);
        }
    });
});